package com.muyu.marketing.team.controller;


import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.req.ActivityTeamUpdReq;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.ActivityTeamDetailResp;
import com.muyu.marketing.domain.resp.ActivityTeamProjectSkuResp;
import com.muyu.marketing.domain.resp.TeamInfoListResp;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.function.Function;

/**
 * 营销团购活动控制层
 *
 * @author DongZeLiang
 * @date 2024-11-20 14:25
 */
@RestController
@RequestMapping("/team")
public class ActivityTeamController {

    @Autowired
    private ActivityTeamInfoService activityTeamInfoService;;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    /**
     * 查询营销团购活动列表
     * @param teamInfoListReq 活动查询入参
     * @return 活动响应结果
     */
    @PostMapping("/list")
    public Result<TableDataInfo<TeamInfoListResp>> list(@RequestBody TeamInfoListReq teamInfoListReq) {
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = activityTeamInfoService.query(teamInfoListReq.buildQueryModel());
        List<TeamInfoListResp> respList = tableDataInfo.getRows().stream().map(TeamInfoListResp::listModelBuild).toList();
        return Result.success(
                new TableDataInfo<>(){{
                    setRows(respList);
                    setTotal(tableDataInfo.getTotal());
                }}
        );
    }


    /**
     * 添加团购活动
     * @param activityTeamInfoSaveReq 添加请求对象
     * @return 结果集
     */
    @PostMapping
    public Result<String> save(@RequestBody ActivityTeamInfoSaveReq activityTeamInfoSaveReq) {
        activityTeamInfoService.save(ActivityTeamInfoAddModel.addReqBuild(activityTeamInfoSaveReq));
        return Result.success();
    }


    @GetMapping("selectById/{id}")
    public ActivityTeamDetailResp selectById(@PathVariable Long id){
        ActivityTeamDetailRespModel respModel = activityTeamInfoService.findById(id);
        ActivityTeamDetailResp activityTeamDetailResp = ActivityTeamDetailResp.teamDetailResp(
                respModel, teamId -> ActivityTeamProjectSkuResp.teamProjectSkuResp(activityTeamProductSkuInfoService.selectByTeamId(teamId))
        );
        return activityTeamDetailResp;
    }

    @PutMapping
    public Result upd(@RequestBody ActivityTeamUpdReq activityTeamUpdReq){
        activityTeamInfoService.upd(ActivityTeamUpdModel.activityTeamUpdReq(activityTeamUpdReq));
        return Result.success();
    }
}
